@@ -452,9 +452,9 @@ void initializeStanControlFromExpression(StanControl& control, SEXP controlExpr)
 #endif
 
 StanSampler::StanSampler(StanModel& stanModel, const StanControl& stanControl, int chain_id, int num_warmup, int verbose) :
-  c_out(verbose >  0 ? rstan::io::rcout : nullout),
-  c_err(verbose >= 0 ? rstan::io::rcerr : nullout),
-  logger(c_out, c_out, c_out, c_err, c_err),
+  c_out(verbose >  0 ? &rstan::io::rcout : &nullout),
+  c_err(verbose >= 0 ? &rstan::io::rcerr : &nullout),
+  logger(*c_out, *c_out, *c_out, *c_err, *c_err),
   diagnostic_writer(diagnostic_stream, "# "),
   init_context_ptr(new stan::io::empty_var_context()),
   init_writer("init"),
@@ -510,6 +510,13 @@ StanSampler::StanSampler(StanModel& stanModel, const StanControl& stanControl, i
   }
 }
 
+void StanSampler::setVerbose(int level)
+{
+  c_out = level >  0 ? &rstan::io::rcout : &nullout;
+  c_err = level >= 0 ? &rstan::io::rcerr : &nullout;
+  new (&logger) stan::callbacks::stream_logger(*c_out, *c_out, *c_out, *c_err, *c_err);
+}
+
 void setStanOffset(StanModel& model, const double* offset)
 {
   model.set_offset(offset);
